require 'logger'
require 'beanstalk_farmer/errors'

##
# Automatically loads classes as needed, provides logger, etc.
module BeanstalkFarmer
  autoload :Config,  'beanstalk_farmer/config'
  autoload :Service, 'beanstalk_farmer/service'
  autoload :DSL,     'beanstalk_farmer/dsl'
  autoload :Job,     'beanstalk_farmer/job'
  autoload :Runner,  'beanstalk_farmer/runner'

  # @private
  def logger
    Config.logger
  end

  # @return [Logger] the logger, defaulting to a STDOUT logger
  def self.logger
    Config.logger
  end

  # @yield [BeanstalkFarmer::Config] a block that allows for convenient configuration
  #
  # @example Usage
  #   BeanstalkFarmer.config do |c|
  #     c.logger = Rails.logger
  #   end
  def self.config
    yield Config
  end
end
