require 'multi_json'

module BeanstalkFarmer
  # Manages configuration settings and defaults.
  #
  # @example
  #   BeanstalkFarmer::Config.logger = Rails.logger
  #   BeanstalkFarmer::Config.json_engine = :yajl
  module Config
    extend self

    DEFAULT_HOST = '0.0.0.0'
    DEFAULT_PORT = 11300

    attr_accessor :settings
    @settings = {}

    # Define a configuration option with a default.
    #
    # @example Define the option.
    #   Config.option(:persist_in_safe_mode, :default => false)
    #
    # @param [Symbol] name The name of the configuration option.
    # @param [Hash] options Extras for the option.
    #
    # @option options [Object] :default The default value.
    #
    # @note Copied from Mongoid. Thank you!
    #
    # @private
    def option(name, options = {})
      define_method(name) do
        settings.has_key?(name) ? settings[name] : options[:default]
      end
      define_method("#{name}=") { |value| settings[name] = value }
      define_method("#{name}?") { send(name) }
    end

    option :host, default: DEFAULT_HOST

    option :port, default: DEFAULT_PORT

    option :logger, default: ::Logger.new($stdout)

    # @see MultiJson.default_engine
    def json_engine
      MultiJson.engine
    end

    # @see MultiJson.engine=
    def json_engine=(json_engine)
      MultiJson.engine = json_engine
    end
  end
end
